@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright © kakkarja (K A K)
+# Copyright © karjakak (K A K)
 
 from .AttSet import AttSet
 import os
 import shutil
 from subprocess import Popen, PIPE
 import argparse
+from datetime import datetime as dt
+from Clien import clien
+import sys
 
 # Reference:
 #stackoverflow.com/.../constantly-print-subprocess-output-while-process-is-running
@@ -15,14 +18,18 @@ def tokfile(token: str = None):
     
     pth = os.path.join(os.environ['USERPROFILE'], '.pypirc')
     ky = None
+    vr = "TOKEN_PYPI"
     if token:
         if token == 'd':
             if '.pypirc' in os.listdir(os.environ['USERPROFILE']):
-                a = AttSet(pth, True)
-                for i in [a.FILE_ATTRIBUTE_HIDDEN, a.FILE_ATTRIBUTE_SYSTEM]:
+                a = AttSet(pth)
+                for i in [a.FILE_ATTRIBUTE_HIDDEN, 
+                          a.FILE_ATTRIBUTE_SYSTEM,
+                          a.FILE_ATTRIBUTE_READONLY,
+                         ]:
                     a.set_file_attrib(i)
                 os.remove(pth)
-                print('Removed') 
+                print('Token Removed') 
             else:
                 print('Nothing to remove, token not created yet!')
         else:
@@ -33,59 +40,78 @@ def tokfile(token: str = None):
         root.withdraw()
         root.update()
         gtt = simpledialog.askstring('', 'Token:', parent = root, show = '*')
-        ask = None
-        if gtt and os.getenv('TOKEN_PYPI') == None:
-            ask = messagebox.askyesno('', 'Do you want set token to variable environment?', parent = root)
-        root.destroy()
         if gtt:
+            ask = messagebox.askyesno('', 'Do you want set token to variable environment?', parent = root)
+            root.destroy()            
             if ask:
-                pnam = f'setx TOKEN_PYPI {gtt}'
-                with Popen(pnam, stdout = PIPE, bufsize = 1, universal_newlines = True, text = True) as p:
-                    for line in p.stdout:
-                        print(line, end='')
-                print('var: TOKEN_PYPI')
+                clien.cmsk(gtt, vr, vr.lower().replace('_',''))
+                print('Please restart Console for var to work!')
+                sys.exit()
+            else:
+                print('Token not set to variable environment!')
             ky = gtt
+        else:
+            print('No token, aborted!')
             
     if ky:
-        f = f'[pypi]\nusername = __token__\npassword = {ky}'
-        if not '.pypirc' in os.listdir(os.environ['USERPROFILE']):
-            with open(pth, 'w') as tkn:
-                tkn.write(f)
-            a = AttSet(pth, True)
-            for i in [a.FILE_ATTRIBUTE_HIDDEN, a.FILE_ATTRIBUTE_SYSTEM]:
-                a.set_file_attrib(i)
-            print('Create')
+        if os.getenv(vr) != None and os.environ[vr] == ky:
+            ky = clien.reading(ky, vr.lower().replace('_', ''))
+            f = f'[pypi]\nusername = __token__\npassword = {ky}'
+            if not '.pypirc' in os.listdir(os.environ['USERPROFILE']):
+                with open(pth, 'w') as tkn:
+                    tkn.write(f)
+                a = AttSet(pth, True)
+                for i in [a.FILE_ATTRIBUTE_HIDDEN, 
+                          a.FILE_ATTRIBUTE_SYSTEM,
+                          a.FILE_ATTRIBUTE_READONLY,
+                         ]:
+                    a.set_file_attrib(i)
+                print('Token created')
+            else:
+                print('Nothing to create, token already created!')
         else:
-            print('Nothing to create, token already created!')
-    else:
-        print('Creating token for publishing is aborted!!!')
+            print('Please set environment variable first!')
 
 def build(path: str):
     # Build egg info, build, dist for upload to PyPI.
+    # When rebuild, existing ones will be removed auto or manually by user.
     
     if os.path.isdir(path):
         os.chdir(path)
-        folds = [f for i in ['dist', 'build','.egg-info'] for f in os.listdir(path) if i in f]
-        if len(folds) == 3:
-            for i in folds:
-                shutil.rmtree(i, ignore_errors = True)
-            pnam = f'py -m build'
-            with Popen(pnam, stdout = PIPE, bufsize = 1, universal_newlines = True, text = True) as p:
-                for line in p.stdout:
-                    print(line, end='')
+        fda = 'Archive_' + path.rpartition("\\")[2] 
+        if fda not in os.listdir():
+            os.mkdir(fda)
+        folds = [f for i in ['build', 'dist', '.egg-info'] for f in os.listdir() if i in f]
+        if folds:
+            fda = os.path.join(fda, f'{str(dt.timestamp(dt.now())).replace(".", "_")}')
+            os.mkdir(fda)
+            try:
+                for i in folds:
+                    shutil.move(i, fda)
+            except Exception as e:
+                print(e)
+                print(f'Please remove {folds} manually!')
+                os.startfile(path)
+                sys.exit()
+        pnam = f'py -m build'
+        with Popen(pnam, stdout = PIPE, bufsize = 1, universal_newlines = True, text = True) as p:
+            for line in p.stdout:
+                print(line, end='')            
                 
 def publish(path: str):
     # Upload to PyPI.
     
     os.chdir(os.environ['USERPROFILE'])
+    pnam = None
     if '.pypirc' in os.listdir():
         pnam = f'py -m twine upload "{path}"'
-        with Popen(pnam, stdout = PIPE, bufsize = 1, universal_newlines = True, text = True) as p:
-            for line in p.stdout:
-                print(line, end='')
     else:
-        tokfile()
-        pnam = f'py -m twine upload "{path}"'
+        if os.getenv('TOKEN_PYPI') == None:
+            print('Please create environment variable first for token!')
+        else:
+            tokfile(os.environ['TOKEN_PYPI'])
+            pnam = f'py -m twine upload "{path}"'
+    if pnam:
         with Popen(pnam, stdout = PIPE, bufsize = 1, universal_newlines = True, text = True) as p:
             for line in p.stdout:
                 print(line, end='')
